# Flamingo Commerce Graphql API

In Flamingo Commerce most of the modules provide GraphQL schemas and the corresponding resolvers and modifiers.

## Usage

To enable graphql in your project follow the documentations in the "graphql" Flamingo module. (See [GraphQL Module](https://docs.flamingo.me/3.%20Flamingo%20Modules/graphql.html))

## Examples

See the following example queries to get a feeling on how the Flamingo Commerce GraphQL schema can be used:

### Receiving Cart
```json
query cartexample {
  Commerce_Cart {
    cart {
      id
      itemCount
      deliveries {
        deliveryInfo {
          code
        }
        cartitems {
          qty
          productName
        }
      }
    }
  }
}
```

### Adding a product to Cart
```json
mutation add {
  Commerce_AddToCart(marketplaceCode: "awesome-retailer_7409939", qty: 10, deliveryCode: "delivery") {
    cart {
      id
    }
}
}
```

### Receiving Products

```json
query productssimple {
  Commerce_Product(marketplaceCode: "awesome-retailer_7409939") {
   ...productData
  }
}


fragment productData on Commerce_Product {
baseData {
      title
    }
  isSaleable
  saleableData {
    activePrice {
      default {
        amount
        currency
      }
      discounted {
        amount
        currency
      }
      discountText
      isDiscounted
    }
  }
}
```

This example uses GraphQL Fragments to share the common set of properties.

#### Typed Products

Product is an interface and there can be many types implementing it. 
You can modify the query like this:

```json
query productstyped {
  Commerce_Product(marketplaceCode: "awesome-retailer_7409939") {
   __typename
    ... on Commerce_SimpleProduct {
      ...productData
    }
  }
}
```